@@ -9,7 +9,7 @@
 #include	<file.h>
 #endif
 
-static char *rcsid = "$Id: atari.c,v 1.5 1996/01/03 23:08:28 david Exp $";
+static char *rcsid = "$Id: atari.c,v 1.7 1996/01/26 23:09:52 david Exp $";
 
 #define FALSE   0
 #define TRUE    1
@@ -135,9 +135,6 @@ void Install_EmuOS (void)
       Atari800_Exit (FALSE);
     }
 
-  SetRAM (0x0000, 0xffff);
-  SetHARDWARE (0xd000, 0xd7ff);
-
   sprintf (filename, "%s/charset.dat", atari_library);
   status = load_image (filename, 0xe000, 0x0400);
   if (!status)
@@ -165,6 +162,8 @@ void Install_EmuOS (void)
 	}
     }
 
+  SetRAM (0x0000, 0xffff);
+  SetHARDWARE (0xd000, 0xd7ff);
   SetROM (0xd800, 0xffff);
 }
 
@@ -173,11 +172,26 @@ main (int argc, char **argv)
   int error;
   int diskno = 1;
   int i;
+  int j;
   char *ptr;
 
   ptr = getenv ("ATARI_LIBRARY");
   if (ptr) atari_library = ptr;
 
+  for (i=j=1;i<argc;i++)
+    {
+      if (strcmp(argv[i],"-atari") == 0)
+	machine = Atari;
+      else if (strcmp(argv[i],"-xl") == 0)
+	machine = AtariXL;
+      else if (strcmp(argv[i],"-xe") == 0)
+	machine = AtariXE;
+      else
+	argv[j++] = argv[i];
+    }
+
+  argc = j;
+
   Atari800_Initialise (&argc, argv);
   Device_Initialise (&argc, argv);
 
@@ -187,19 +201,7 @@ main (int argc, char **argv)
 
   for (i=1;i<argc;i++)
     {
-      if (strcmp(argv[i],"-atari") == 0)
-	{
-	  machine = Atari;
-	}
-      else if (strcmp(argv[i],"-xl") == 0)
-	{
-	  machine = AtariXL;
-	}
-      else if (strcmp(argv[i],"-xe") == 0)
-	{
-	  machine = AtariXE;
-	}
-      else if (strcmp(argv[i],"-ffp") == 0)
+      if (strcmp(argv[i],"-ffp") == 0)
 	{
 	  ffp_patch = TRUE;
 	}
@@ -325,8 +327,6 @@ main (int argc, char **argv)
 
   if (sio_patch)
     add_esc (0xe459, ESC_SIOV);
-
-  Escape = Atari800_ESC;
 /*
  * ================================
  * Install requested ROM cartridges
@@ -761,7 +761,7 @@ DSKIN ()
 
 #define CDTMV1 0x0218
 
-void Atari800_ESC (UBYTE esc_code)
+void Escape (UBYTE esc_code)
 {
   int addr;
 
