@@ -41,7 +41,7 @@
 #include	<stdio.h>
 #include	<stdlib.h>
 
-static char *rcsid = "$Id: cpu.c,v 1.2 1995/12/30 22:11:08 david Exp $";
+static char *rcsid = "$Id: cpu.c,v 1.4 1996/01/27 21:49:18 david Exp $";
 
 #define	FALSE	0
 #define	TRUE	1
@@ -72,16 +72,31 @@ static UBYTE	I;
 static UBYTE	Z;	/* zero (0) or non-zero (1) */
 static UBYTE	C;	/* zero (0) or non-zero (1) */
 
-UBYTE	memory[65536];
-UBYTE	attrib[65536];
+#define RAM 0
+#define ROM 1
+#define HARDWARE 2
 
-int	INTERRUPT;
-int	ncycles;
+#define MEMORY_ATTRIBUTES
 
-void  (*Escape)   (UBYTE byte);
+UBYTE memory[65536];
 
+static int ram_below = 0xa000;
+static int ram_above = 0xffff;
+
+int INTERRUPT;
+int ncycles;
+
+#ifdef MEMORY_ATTRIBUTES
+static UBYTE attrib[65536];
 #define	GetByte(addr)		((attrib[addr] == HARDWARE) ? Atari800_GetByte(addr) : memory[addr])
 #define	PutByte(addr,byte)	if (attrib[addr] == RAM) memory[addr] = byte; else if (attrib[addr] == HARDWARE) Atari800_PutByte(addr,byte);
+#define isRAM(addr) (attrib[addr] == RAM)
+#else
+#define GetByte(addr) ((addr < 0xd000) ? memory[addr] : ((addr > 0xd7ff) ? memory[addr] : Atari800_GetByte(addr)))
+#define PutByte(addr,byte) if ((int)addr < ram_below) memory[addr] = byte; else if ((int)addr > ram_above) memory[addr] = byte; else Atari800_PutByte(addr,byte);
+#define isRAM(addr) ((addr < ram_below) || (addr > ram_above))
+#endif
+
 #define	GetWord(addr)		((GetByte(addr+1) << 8) | GetByte(addr))
 
 /*
@@ -164,6 +179,42 @@ void CPU_Reset (void)
 	s_PC = (GetByte(0xfffd) << 8) | GetByte(0xfffc);
 }
 
+void SetRAM (int addr1, int addr2)
+{
+#ifdef MEMORY_ATTRIBUTES
+  int	i;
+
+  for (i=addr1;i<=addr2;i++)
+    {
+      attrib[i] = RAM;
+    }
+#endif
+}
+
+void SetROM (int addr1, int addr2)
+{
+#ifdef MEMORY_ATTRIBUTES
+  int	i;
+
+  for (i=addr1;i<=addr2;i++)
+    {
+      attrib[i] = ROM;
+    }
+#endif
+}
+
+void SetHARDWARE (int addr1, int addr2)
+{
+#ifdef MEMORY_ATTRIBUTES
+  int	i;
+
+  for (i=addr1;i<=addr2;i++)
+    {
+      attrib[i] = HARDWARE;
+    }
+#endif
+}
+
 #define PHP data =  (N & 0x80) ? 0x80 : 0; \
             data |= V ? 0x40 : 0; \
             data |= 0x20; \
@@ -1744,7 +1795,7 @@ int GO (void)
 
     opcode_ce: /* DEC abcd */
       ABSOLUTE;
-      if (attrib[addr] == RAM)
+      if (isRAM(addr))
 	Z = N = --memory[addr];
       goto next;
 
@@ -1799,7 +1850,7 @@ int GO (void)
 
     opcode_de: /* DEC abcd,x */
       ABSOLUTE_X;
-      if (attrib[addr] == RAM)
+      if (isRAM(addr))
 	Z = N = --memory[addr];
       goto next;
 
@@ -1856,7 +1907,7 @@ int GO (void)
 
     opcode_ee: /* INC abcd */
       ABSOLUTE;
-      if (attrib[addr] == RAM)
+      if (isRAM(addr))
 	Z = N = ++memory[addr];
       goto next;
 
@@ -1903,7 +1954,7 @@ int GO (void)
 
     opcode_fe: /* INC abcd,x */
       ABSOLUTE_X;
-      if (attrib[addr] == RAM)
+      if (isRAM(addr))
 	Z = N = ++memory[addr];
       goto next;
 
